@@ -18,27 +18,37 @@
  * Authors : Carlos Garca Campos <carlosgc@gnome.org>
  */
 
-
 #include <config.h>
 
 #include <gnome.h>
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 #include <glade/glade.h>
+#include <string.h>
 
 #include "cpufreq-applet.h"
 #include "cpufreq-prefs.h"
+#include "cpufreq-popup.h"
 #include "cpufreq.h"
 
-static void     cpufreq_applet_preferences_dialog (BonoboUIComponent *uic, CPUFreqApplet *applet);
-static void     cpufreq_applet_help_cb  (BonoboUIComponent *uic, CPUFreqApplet *applet);
-static void     cpufreq_applet_about_cb (BonoboUIComponent *uic, CPUFreqApplet *applet);
-static gint     cpufreq_applet_get_max_cpu (void);
-static void     cpufreq_applet_destroy  (CPUFreqApplet *applet);
-static void     cpufreq_setup_widgets   (CPUFreqApplet *applet);
-static GtkWidget *cpufreq_applet_new    (CPUFreqApplet *applet);
-static gboolean cpufreq_applet_fill     (CPUFreqApplet *applet);
-static gboolean cpufreq_applet_factory  (CPUFreqApplet *applet, const gchar *iid, gpointer gdata);
+static void       cpufreq_applet_preferences_dialog (BonoboUIComponent *uic, CPUFreqApplet *applet);
+static void       cpufreq_applet_help_cb            (BonoboUIComponent *uic, CPUFreqApplet *applet);
+static void       cpufreq_applet_about_cb           (BonoboUIComponent *uic, CPUFreqApplet *applet);
+static gint       cpufreq_applet_get_max_cpu        (void);
+static void       cpufreq_applet_pixmap_set_image   (CPUFreqApplet *applet);
+static void       cpufreq_applet_destroy            (CPUFreqApplet *applet);
+static void       cpufreq_setup_widgets             (CPUFreqApplet *applet);
+static void       cpufreq_size_allocate_cb          (PanelApplet *pa, GtkAllocation *allocation,
+										   gpointer gdata);
+static void       cpufreq_change_orient_cb          (PanelApplet *pa, PanelAppletOrient orient,
+										   gpointer gdata);
+static void       cpufreq_background_changed        (PanelApplet *pa, PanelAppletBackgroundType type,
+										   GdkColor *color, GdkPixmap *pixmap,
+										   CPUFreqApplet *applet);
+static GtkWidget *cpufreq_applet_new                (CPUFreqApplet *applet);
+static gboolean   cpufreq_applet_fill               (CPUFreqApplet *applet);
+static gboolean   cpufreq_applet_factory            (CPUFreqApplet *applet, const gchar *iid,
+										   gpointer gdata);
 
 static const BonoboUIVerb cpufreq_menu_verbs [] = {
 	   BONOBO_UI_UNSAFE_VERB ("CPUFreqAppletPreferences",
@@ -117,8 +127,7 @@ cpufreq_applet_help_cb (BonoboUIComponent *uic, CPUFreqApplet *applet)
 static void
 cpufreq_applet_about_cb (BonoboUIComponent *uic, CPUFreqApplet *applet)
 {
-	   static GtkWidget *about = NULL;
-	   GdkPixbuf *pixbuf;
+	   GdkPixbuf   *pixbuf;
 	   const gchar *authors[] = {
 			 "Carlos Garcia Campos <carlosgc@gnome.org>",
 			 " ",
@@ -142,7 +151,8 @@ cpufreq_applet_about_cb (BonoboUIComponent *uic, CPUFreqApplet *applet)
 			 return;
 	   }
 
-	   pixbuf = gdk_pixbuf_new_from_file (ICONDIR"/gnome-cpufreq-applet/cpufreq-applet.png", NULL);
+	   pixbuf = gdk_pixbuf_new_from_file_at_size (ICONDIR"/gnome-cpufreq-applet/cpufreq-applet.png",
+										 48, 48, NULL);
 
 	   applet->about_dialog = gnome_about_new (
 			 _("CPU Frequency Scaling Monitor"),
@@ -151,7 +161,8 @@ cpufreq_applet_about_cb (BonoboUIComponent *uic, CPUFreqApplet *applet)
 			 _("This utility shows the current CPU Frequency Scaling."),
 			 authors,
 			 documenters,
-			 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+			 g_ascii_strcasecmp (
+				    translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 			 pixbuf);
 
 	   g_object_unref (pixbuf);
@@ -171,7 +182,7 @@ cpufreq_applet_about_cb (BonoboUIComponent *uic, CPUFreqApplet *applet)
 static gint
 cpufreq_applet_get_max_cpu ()
 {
-	   gint mcpu = -1;
+	   gint   mcpu = -1;
 	   gchar *file = NULL;
 
 	   do {
@@ -185,6 +196,8 @@ cpufreq_applet_get_max_cpu ()
 	   if (mcpu >= 0)
 			 return mcpu;
 
+	   mcpu = -1;
+	   file = NULL;
 	   do {
 			 if (file) g_free (file);
 			 mcpu ++;
@@ -199,79 +212,113 @@ cpufreq_applet_get_max_cpu ()
 			 return 0;
 }
 
-void
-cpufreq_applet_update (CPUFreqApplet *applet, CPUFreqMode mode)
+static void
+cpufreq_applet_pixmap_set_image (CPUFreqApplet *applet)
 {
+	   gint   perc, image;
+	   gchar *text_perc;
 	   gchar *pixmaps[] = {
 			 ICONDIR"/gnome-cpufreq-applet/cpufreq-25.png",
 			 ICONDIR"/gnome-cpufreq-applet/cpufreq-50.png",
 			 ICONDIR"/gnome-cpufreq-applet/cpufreq-75.png",
 			 ICONDIR"/gnome-cpufreq-applet/cpufreq-100.png",
+			 ICONDIR"/gnome-cpufreq-applet/cpufreq-na.png",
 			 NULL };
-	   gint perc, image;
-	   gchar *text_label, *text_tip;
 
-	   g_return_if_fail (PANEL_IS_APPLET (PANEL_APPLET (applet)));
-	   
-	   switch (applet->show_mode) {
-	   case MODE_FREQUENCY:
-			 if (applet->show_unit) {
-				    text_label = g_strdup_printf ("%s %s", applet->freq, applet->unit);
-				    gtk_label_set_text (GTK_LABEL (applet->label), text_label);
-				    g_free (text_label);
-			 } else {
-				    gtk_label_set_text (GTK_LABEL (applet->label), applet->freq);
-			 }
-			 
-			 gtk_widget_show (applet->label);
-			 break;
-	   case MODE_PERCENTAGE:
-			 gtk_label_set_text (GTK_LABEL (applet->label), applet->perc);
-			 gtk_widget_show (applet->label);
-			 break;
-	   case MODE_NONE:
-			 gtk_widget_hide (applet->label);
-	   }
-
-	   if (mode == POWERSAVE) {
-			 text_tip = g_strdup_printf ("CPU %d - Powersave\n%s %s (%s)",
-								    applet->cpu, applet->freq,
-								    applet->unit, applet->perc);
-	   } else if (mode == PERFORMANCE) {
-			 text_tip = g_strdup_printf ("CPU %d - Performance\n%s %s (%s)",
-								    applet->cpu, applet->freq,
-								    applet->unit, applet->perc);
+	   if (!applet->perc) {
+			 image = 4;
 	   } else {
-			 text_tip = g_strdup_printf ("CPU %d - Userspace\n%s %s (%s)",
-								    applet->cpu, applet->freq,
-								    applet->unit, applet->perc);
+			 text_perc = g_strndup (applet->perc, strlen (applet->perc) - 1);
+			 perc = atoi (text_perc);
+			 g_free (text_perc);
+	   
+			 /* 0-29   -> 25%	
+			  * 30-69  -> 50%	
+			  * 70-89  -> 75%	
+			  * 90-100 -> 100%	
+			  */
+			 if (perc < 30)
+				    image = 0;
+			 else if ((perc >= 30) && (perc < 70))
+				    image = 1;
+			 else if ((perc >= 70) && (perc < 90))
+				    image = 2;
+			 else
+				    image = 3;
 	   }
-	   
-	   gtk_tooltips_set_tip (applet->tips, GTK_WIDGET (applet), text_tip, NULL);
-	   g_free (text_tip);
 
-	   perc = atoi (g_strndup (applet->perc, strlen (applet->perc) - 1));
-	   /* 0-29   -> 25%
-	    * 30-69  -> 50%
-	    * 70-89  -> 75%
-	    * 90-100 -> 100%
-	    */
-	   if (perc < 30)
-			 image = 0;
-	   else if ((perc >= 30) && (perc < 70))
-			 image = 1;
-	   else if ((perc >= 70) && (perc < 90))
-			 image = 2;
-	   else
-			 image = 3;
-	   
 	   if (applet->pixbufs[image] == NULL) {
-			 applet->pixbufs[image] = gdk_pixbuf_new_from_file (pixmaps[image], NULL);
+			 applet->pixbufs[image] = gdk_pixbuf_new_from_file_at_size (pixmaps[image],
+															24, 24, NULL);
 	   }
 
 	   gtk_image_set_from_pixbuf (GTK_IMAGE (applet->pixmap), applet->pixbufs[image]);
+}
+
+void
+cpufreq_applet_update (CPUFreqApplet *applet)
+{
+	   gchar *text_tip, *text_mode;
+	   gchar *governor;
+
+	   g_return_if_fail (PANEL_IS_APPLET (PANEL_APPLET (applet)));
+
+	   if (applet->show_mode != MODE_GRAPHIC) {
+			 if (applet->show_mode == MODE_TEXT) {
+				    gtk_widget_hide (applet->pixmap);
+			 } else {
+				    cpufreq_applet_pixmap_set_image (applet);
+				    gtk_widget_show (applet->pixmap);
+			 }
+
+			 switch (applet->show_text_mode) {
+			 case MODE_TEXT_FREQUENCY:
+				    gtk_label_set_text (GTK_LABEL (applet->label), applet->freq);
+				    gtk_widget_hide (applet->unit_label);
+
+				    break;
+			 case MODE_TEXT_FREQUENCY_UNIT:
+				    gtk_label_set_text (GTK_LABEL (applet->label), applet->freq);
+				    gtk_label_set_text (GTK_LABEL (applet->unit_label), applet->unit);
+				    gtk_widget_show (applet->unit_label);
+
+				    break;
+			 case MODE_TEXT_PERCENTAGE:
+				    gtk_label_set_text (GTK_LABEL (applet->label), applet->perc);
+				    
+				    break;
+			 }
+			 
+			 gtk_widget_show (applet->label);
+	   } else {
+			 cpufreq_applet_pixmap_set_image (applet);
+			 
+			 gtk_widget_show (applet->pixmap);
+			 gtk_widget_hide (applet->label);
+			 gtk_widget_hide (applet->unit_label);
+	   }
 	   
-	   gtk_widget_show (applet->pixmap);
+	   governor = g_strdup (applet->governor);
+	   governor[0] = g_ascii_toupper (governor[0]);
+	   text_mode = g_strdup_printf ("%s\n%s %s (%s)", governor, 
+							  applet->freq, applet->unit, applet->perc);
+	   g_free (governor);
+
+	   if (applet->mcpu == 0)
+			 text_tip = g_strdup_printf ("%s", text_mode);
+	   else
+			 text_tip = g_strdup_printf ("CPU %d - %s", applet->cpu, text_mode);
+
+	   g_free (text_mode);
+	   
+	   gtk_tooltips_set_tip (applet->tips, GTK_WIDGET (applet), text_tip, NULL);
+	   g_free (text_tip);
+}
+
+static void
+free_string (gpointer str, gpointer gdata)
+{
+	   if (str) g_free (str);
 }
 
 static void
@@ -307,6 +354,13 @@ cpufreq_applet_destroy (CPUFreqApplet *applet)
 			 applet->unit = NULL;
 	   }
 
+	   if (applet->available_freqs) {
+			 g_list_foreach (applet->available_freqs,
+						  free_string, NULL);
+			 g_list_free (applet->available_freqs);
+			 applet->available_freqs = NULL;
+	   }
+
 	   if (applet->about_dialog) {
 			 gtk_widget_destroy (applet->about_dialog);
 			 applet->about_dialog = NULL;
@@ -316,21 +370,28 @@ cpufreq_applet_destroy (CPUFreqApplet *applet)
 			 gtk_widget_destroy (applet->prefs);
 			 applet->prefs = NULL;
 	   }
+
+	   if (applet->popup) {
+			 gtk_widget_destroy (applet->popup);
+			 applet->popup = NULL;
+	   }
 }	
 
 static void
 cpufreq_setup_widgets (CPUFreqApplet *applet)
 {
-	   GtkRequisition req;
-	   gint total_size = 4;
-	   gboolean horizontal = FALSE;
-	   gint panel_size;
+	   GtkWidget      *labels_box;
+	   GtkRequisition  req;
+	   gint            total_size = 0;
+	   gboolean        horizontal = FALSE;
+	   gint            panel_size, label_size, unit_label_size, pixmap_size;
+	   gint            size_step = 12;
 
 	   g_return_if_fail (PANEL_IS_APPLET (PANEL_APPLET (applet)));
 	   
-	   panel_size = panel_applet_get_size (PANEL_APPLET (applet));
+	   panel_size = applet->size - 1; /* 1 pixel margin */
 
-	   switch (panel_applet_get_orient (PANEL_APPLET (applet))) {
+	   switch (applet->orient) {
 	   case PANEL_APPLET_ORIENT_LEFT:
 	   case PANEL_APPLET_ORIENT_RIGHT:
 			 horizontal = FALSE;
@@ -344,66 +405,165 @@ cpufreq_setup_widgets (CPUFreqApplet *applet)
 	   if (applet->label)
 			 gtk_widget_destroy (applet->label);
 	   
-	   applet->label = gtk_label_new ("---");
+	   applet->label = gtk_label_new (" --- ");
+	   if (applet->freq)
+			 gtk_label_set_text (GTK_LABEL (applet->label), applet->freq);
 	   
 	   gtk_widget_size_request (applet->label, &req);
-	   if (applet->show_mode != MODE_NONE)
+	   if (applet->show_mode != MODE_GRAPHIC)
 			 gtk_widget_show (applet->label);
 
-	   if (horizontal)
+	   if (horizontal) {
+			 label_size = req.height;
 			 total_size += req.height;
-	   else
+	   } else {
+			 label_size = req.width;
 			 total_size += req.width;
+	   }
+	   
+	   if (applet->unit_label)
+			 gtk_widget_destroy (applet->unit_label);
+				    
+	   applet->unit_label = gtk_label_new ("  ?  ");
+	   if (applet->unit)
+			 gtk_label_set_text (GTK_LABEL (applet->unit_label), applet->unit);
+	   
+	   gtk_widget_size_request (applet->unit_label, &req);
+	   if (applet->show_mode != MODE_GRAPHIC &&
+		  applet->show_text_mode == MODE_TEXT_FREQUENCY_UNIT)
+			 gtk_widget_show (applet->unit_label);
+
+	   if (horizontal) {
+			 unit_label_size = req.height;
+			 total_size += req.height;
+	   } else {
+			 unit_label_size = req.width;
+			 total_size += req.width;
+	   }
 
 	   if (applet->pixmap)
 			 gtk_widget_destroy (applet->pixmap);
-	   
-	   applet->pixmap =
-			 gtk_image_new_from_file (ICONDIR"/gnome-cpufreq-applet/cpufreq-na.png");
+
+	   applet->pixmap = gtk_image_new ();
+
+	   cpufreq_applet_pixmap_set_image (applet);
+
 	   gtk_widget_size_request (applet->pixmap, &req);
-	   gtk_widget_show (applet->pixmap);
+	   if (applet->show_mode != MODE_TEXT)
+			 gtk_widget_show (applet->pixmap);
 
-	   if (horizontal)
+	   if (horizontal) {
+			 pixmap_size = req.height;
 			 total_size += req.height;
-	   else
+	   } else {
+			 pixmap_size = req.width;
 			 total_size += req.width;
-
-	   /* FIXME */
-	   if (applet->show_unit) {
-			 if (horizontal)
-				    total_size += 3;
-			 else
-				    total_size += 3;
 	   }
-	   
+
 	   if (applet->box)
 			 gtk_widget_destroy (applet->box);
 
-	   if (horizontal && (total_size <= panel_size))
-			 applet->box = gtk_vbox_new (FALSE, 2);
-	   else if (horizontal && (total_size > panel_size))
-			 applet->box = gtk_hbox_new (FALSE, 2);
-	   else if (!horizontal && (total_size <= panel_size))
-			 applet->box = gtk_hbox_new (FALSE, 2);
-	   else
-			 applet->box = gtk_vbox_new (FALSE, 2);
+	   if (horizontal) {
+			 labels_box  = gtk_hbox_new (FALSE, 2);
+			 if ((label_size + pixmap_size) <= panel_size)
+				    applet->box = gtk_vbox_new (FALSE, 2);
+			 else
+				    applet->box = gtk_hbox_new (FALSE, 2);
+	   } else {
+			 
+			 if (total_size <= panel_size) {
+				    applet->box = gtk_hbox_new (FALSE, 2);
+				    labels_box  = gtk_hbox_new (FALSE, 2);
+			 } else if ((label_size + unit_label_size) <= (panel_size - size_step)) {
+				    applet->box = gtk_vbox_new (FALSE, 2);
+				    labels_box  = gtk_hbox_new (FALSE, 2);
+			 } else {
+				    applet->box = gtk_vbox_new (FALSE, 2);
+				    labels_box  = gtk_vbox_new (FALSE, 2);
+			 }
+	   }
+			 
+	   gtk_box_pack_start (GTK_BOX (labels_box), applet->label, FALSE, TRUE, 0);
+	   gtk_box_pack_start (GTK_BOX (labels_box), applet->unit_label, FALSE, TRUE, 0);
 
-	   gtk_box_pack_start (GTK_BOX (applet->box), applet->pixmap, TRUE, TRUE, 0);
-	   gtk_box_pack_start (GTK_BOX (applet->box), applet->label, TRUE, TRUE, 0);
+	   gtk_widget_show (labels_box);
+	   
+	   gtk_box_pack_start (GTK_BOX (applet->box), applet->pixmap, FALSE, TRUE, 0);
+	   gtk_box_pack_start (GTK_BOX (applet->box), labels_box, FALSE, TRUE, 0);
 	   
 	   gtk_widget_show (applet->box);
 
-	   gtk_container_add (GTK_CONTAINER (applet), applet->box);
+	   gtk_container_add (GTK_CONTAINER (applet->container), applet->box);
+
+	   gtk_widget_show (applet->container);
 }
 
+static void
+cpufreq_size_allocate_cb (PanelApplet *pa, GtkAllocation *allocation,
+					 gpointer gdata)
+{
+	   CPUFreqApplet *applet;
+
+	   applet = (CPUFreqApplet *) gdata;
+
+	   if ((applet->orient == PANEL_APPLET_ORIENT_LEFT) ||
+		  (applet->orient == PANEL_APPLET_ORIENT_RIGHT)) {
+			 if (applet->size == allocation->width)
+				    return;
+			 applet->size = allocation->width;
+	   } else {
+			 if (applet->size == allocation->height)
+				    return;
+			 applet->size = allocation->height;
+	   }
+
+	   cpufreq_setup_widgets (applet);
+}
+
+static void
+cpufreq_change_orient_cb (PanelApplet *pa, PanelAppletOrient orient, gpointer gdata)
+{
+	   CPUFreqApplet *applet;
+
+	   applet = (CPUFreqApplet *) gdata;
+
+	   applet->orient = orient;
+
+	   cpufreq_setup_widgets (applet);
+}
+
+static void
+cpufreq_background_changed (PanelApplet *pa, PanelAppletBackgroundType type,
+					   GdkColor *color, GdkPixmap *pixmap,
+					   CPUFreqApplet *applet)
+{
+	   GtkRcStyle *rc_style = gtk_rc_style_new ();
+	   
+	   switch (type) {
+	   case PANEL_PIXMAP_BACKGROUND:
+			 gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+			 break;
+	   case PANEL_COLOR_BACKGROUND:
+			 gtk_widget_modify_bg (GTK_WIDGET (applet), GTK_STATE_NORMAL, color);
+			 break;
+	   case PANEL_NO_BACKGROUND:
+			 gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+			 break;
+	   default:
+			 gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+			 break;
+	   }
+
+	   gtk_rc_style_unref (rc_style);
+}
 
 static GtkWidget *
 cpufreq_applet_new (CPUFreqApplet *applet)
 {
 	   AtkObject *atk_obj;
-	   gint i;
-	   gchar *text_tip;
-	   GError *error;
+	   gint       i;
+	   gchar     *text_tip;
+	   GError    *error;
 
 	   panel_applet_add_preferences (PANEL_APPLET (applet),
 							   "/schemas/apps/gnome-cpufreq-applet/prefs", NULL);
@@ -411,57 +571,90 @@ cpufreq_applet_new (CPUFreqApplet *applet)
 	   panel_applet_set_flags (PANEL_APPLET (applet), PANEL_APPLET_EXPAND_MINOR);
 
 	   /* New applet, default values */
-	   applet->cpu = 0;
 	   applet->timeout_handler = 0;
 	   applet->mcpu = cpufreq_applet_get_max_cpu ();
 	   applet->prefs = NULL;
 	   applet->about_dialog = NULL;
+	   applet->available_freqs = NULL;
+	   applet->popup = NULL;
 
 	   error = NULL;
-	   applet->show_mode = panel_applet_gconf_get_int (PANEL_APPLET (applet), "show_mode", &error);
+	   applet->cpu = panel_applet_gconf_get_int (PANEL_APPLET (applet),
+										"cpu", &error);
+
+	   /* In case anything went wrong with gconf, get back to the default */
+	   if (error || applet->cpu < 0) {
+			 applet->cpu = 0;
+			 if (error)
+				    g_error_free (error);
+	   }
+	   
+	   error = NULL;
+	   applet->show_mode = panel_applet_gconf_get_int (PANEL_APPLET (applet),
+											 "show_mode", &error);
 	   
         /* In case anything went wrong with gconf, get back to the default */
-	   if (error || applet->show_mode < MODE_FREQUENCY || applet->show_mode > MODE_NONE) {
-			 applet->show_mode = MODE_FREQUENCY;
+	   if (error || applet->show_mode < MODE_GRAPHIC || applet->show_mode > MODE_BOTH) {
+			 applet->show_mode = MODE_BOTH;
 			 if (error)
 				    g_error_free (error);
 	   }
 
 	   error = NULL;
-	   applet->show_unit = panel_applet_gconf_get_bool (PANEL_APPLET (applet), "show_unit", &error);
+	   applet->show_text_mode = panel_applet_gconf_get_int (PANEL_APPLET (applet),
+												  "show_text_mode", &error);
 
 	   /* In case anything went wrong with gconf, get back to the default */
-	   if (error) {
-			 applet->show_unit = TRUE;
+	   if (error || applet->show_text_mode < MODE_TEXT_FREQUENCY ||
+		  applet->show_text_mode > MODE_TEXT_PERCENTAGE) {
+			 applet->show_text_mode = MODE_TEXT_FREQUENCY_UNIT;
 			 g_error_free (error);
 	   }
 	   
-	   applet->freq = NULL;
-	   applet->perc = NULL;
-	   applet->unit = NULL;
+	   applet->freq     = NULL;
+	   applet->perc     = NULL;
+	   applet->unit     = NULL;
+	   applet->governor = NULL;
 
-	   for (i=0; i<=3; i++)
+	   for (i=0; i<=4; i++)
 			 applet->pixbufs[i] = NULL;
 
 	   applet->tips = gtk_tooltips_new ();
 	   g_object_ref (G_OBJECT (applet->tips));
 
-	   cpufreq_setup_widgets (applet);
-	   
 	   g_signal_connect (G_OBJECT (applet), "destroy",
 					 G_CALLBACK (cpufreq_applet_destroy),
 					 NULL);
+	   g_signal_connect (G_OBJECT (applet), "button_press_event",
+					 G_CALLBACK (cpufreq_popup_show),
+					 NULL);
+	   g_signal_connect (G_OBJECT (applet), "size_allocate",
+					 G_CALLBACK (cpufreq_size_allocate_cb),
+					 (gpointer) applet);
+	   g_signal_connect (G_OBJECT (applet), "change_orient",
+					 G_CALLBACK (cpufreq_change_orient_cb),
+					 (gpointer) applet);
+	   g_signal_connect (G_OBJECT (applet), "change_background",
+					 G_CALLBACK (cpufreq_background_changed),
+					 (gpointer) applet);
+
+	   applet->container = gtk_alignment_new (0.5, 0.5, 0, 0);
+	   gtk_container_add (GTK_CONTAINER (applet), applet->container);
+	   
+	   applet->size = panel_applet_get_size (PANEL_APPLET (applet));
+	   applet->orient = panel_applet_get_orient (PANEL_APPLET (applet));
+	   
+	   cpufreq_setup_widgets (applet);
 
 	   /* Setup the menus */
 	   panel_applet_setup_menu_from_file (PANEL_APPLET (applet),
-								   NULL,
+								   DATADIR,
 								   "GNOME_CPUFreqApplet.xml",
 								   NULL,
 								   cpufreq_menu_verbs,
 								   applet);
 
-	   /* Not in 2.4 */
-	   /*if (panel_applet_get_locked_down (PANEL_APPLET (applet))) {
+	   if (panel_applet_get_locked_down (PANEL_APPLET (applet))) {
 			 BonoboUIComponent *popup_component;
 			 
 			 popup_component = panel_applet_get_popup_component (PANEL_APPLET (applet));
@@ -470,12 +663,13 @@ cpufreq_applet_new (CPUFreqApplet *applet)
 									 "/commands/CPUFreqPreferences",
 									 "hidden", "1",
 									 NULL);
-									 }*/
-	   
+	   }
 
 	   if (g_file_test ("/sys/devices/system/cpu/cpu0/cpufreq", G_FILE_TEST_EXISTS)) { /* 2.6 kernel */
+			 applet->iface = IFACE_SYSFS;
 			 applet->timeout_handler = g_timeout_add (1000, cpufreq_get_from_sysfs, (gpointer) applet);
 	   } else if (g_file_test ("/proc/cpufreq", G_FILE_TEST_EXISTS)) { /* 2.4 kernel */
+			 applet->iface = IFACE_PROCFS;
 			 applet->timeout_handler = g_timeout_add (1000, cpufreq_get_from_procfs, (gpointer) applet);
 	   } else if (g_file_test ("/proc/cpuinfo", G_FILE_TEST_EXISTS)) {
 			 /* If there is no cpufreq support it shows only the cpu frequency,
@@ -488,7 +682,8 @@ cpufreq_applet_new (CPUFreqApplet *applet)
 									 "/sys or /proc file systems are mounted.\n"
 									 "Now the applet will only show the current "
 									 "cpu frequency.");
-			 
+
+			 applet->iface = IFACE_CPUINFO;
 			 if (cpufreq_get_from_procfs_cpuinfo (applet)) {
 				    text_tip = g_strdup_printf ("CPU %d - cpufreq Not Supported\n%s (%s)",
 										  applet->cpu, applet->freq, applet->perc);
@@ -510,11 +705,6 @@ cpufreq_applet_new (CPUFreqApplet *applet)
 			 atk_object_set_description (atk_obj, _("This utility shows the current CPU Frequency"));
 	   }
 
-	   g_signal_connect (G_OBJECT (applet), "change_size",
-					 G_CALLBACK (cpufreq_setup_widgets), applet);
-	   g_signal_connect (G_OBJECT (applet), "change_orient",
-					 G_CALLBACK (cpufreq_setup_widgets), applet);
-
 	   return GTK_WIDGET (applet);
 }
 
